/**
 * @file config.h
 * @brief Numerical configuration parameters
 * @author Lukas Hostettler
 * @author Roger Fong
 * @version 1.1
 * @date 2015-12-15
 */

#ifndef CONFIG_H
#define CONFIG_H

/*
 * Values used for memory allocation
 */

// Maximal number of admissible image segments
#define MAX_NUM_IMAGE_SEGMENTS      2000

// More than 1000 points seem to be unnecessary ;D
#define MAX_NUM_POINTS              1000

// Maximal sidelength of the visible grid
#define MAX_GRID_LENGTH             30

// Bit-shift of the representation in grid-coordinates
#define SHIFT_POWER 10

// Side length of the decoding grid
#define DECODING_GRID_LENGTH 8
/*
 * Image processing parameters
 * CAUTION: These values are tuned for our camera; must be tuned again for another one
 */

// All image segments containing less pixels are considered noise
#define MIN_PIXELS_PER_SEGMENT      2

// Offset between image-mean and the threshold for the image segmentation
#define THRESHOLD_OFFSET            25

// Camera's radial distortion
#define CAMERA_DISTORTION_PARAMETER -0.032

/*
 * Algorithm parameters
 */

// Extent of the probabilities: 0-PROB_MAX
#define PROB_MAX                    1024LL

// Gives the integer output as FIXED_RESULT_DIVISOR*gridCoordinate
#define FIXED_RESULT_DIVISOR        100

// Anti-windup-reset for the rotation decoder; the greater it is the less likely the current rotation state will be changed
#define ROTATION_DECODER_AWR        15

// Usual offset of the points from the grid positions; gridspacing assumed to be 1024 (this does not correspond to 1/6 but rather about 1/4, it is observed to behave better)
#define MEAN_OFFSET                 250

// Fixed divisor for image coordinates, must be chosen in order to avoid overflow
#define SUBDIVISION                 64

// Initial length of the axis estimate
#define AXIS_INIT_LENGTH            (20*SUBDIVISION)


#endif //CONFIG_H

