/**
 * @file imgsegment.h
 * @brief Header of the ImgSegment structure
 * @author Lukas Hostettler
 * @author Roger Fong
 * @version 1.1
 * @date 2015-12-15
 */

#ifndef IMGSEGMENT_H
#define IMGSEGMENT_H

#ifdef __cplusplus
extern "C" {
#endif

/// @brief Describes a blob
typedef struct _ImgSegment {
    unsigned int n; ///< Number of pixels added
    unsigned int x; ///< Sum of all x values
    unsigned int y; ///< Sum of all y values
}ImgSegment;

/// @brief  Creates and initialises an ImgSegment
/// @var    x       Sum of pixels in x-direction
/// @var    y       Sum of pixels in y-direction
/// @var    n       Number of summands
/// @return ImageSegment With the initial values x,y,n
ImgSegment imgSegmentCreate(unsigned int x, unsigned int y, unsigned int n);

/// @brief  Adds all the values of seg2 to seg1
/// @var    seg1    Pointer to single Image segment (the pointed value will be modified)
/// @var    seg2    ImgSegment structure
void imgSegmentAdd(ImgSegment* seg1, ImgSegment seg2);

/// @brief  Adds a single pixel to seg and increments pixel-counter
/// @var    seg     Pointer to ImgSegment structure
/// @var    x       Pixel x-coordinate
/// @var    y       Pixel y-coordinate
void imgSegmentAddPixel(ImgSegment* seg, unsigned int x, unsigned int y);

#ifdef __cplusplus
}
#endif

#endif // IMGSEGMENT_H
