/**
 * @file intlist.c
 * @brief Handles IntList median calculation implementation
 * @author Lukas Hostettler
 * @author Roger Fong
 * @version 1.1
 * @date 2015-12-15
 */

#ifdef __cplusplus
extern "C" {
#endif

#include "compiler.h"
#include "config.h"

#include "intlist.h"

#pragma GCC push_options
#pragma GCC optimize ("O1")

/****************************************
 * Helper function forward declarations *
 ****************************************/
/// Attention! This function changes the order of the content of list
/// @brief  finds the value of the n-th-biggest element in a list
/// @var    list    pointer to an array of integers (will be changed!)
/// @var    left    leftmost index in the array to consider
/// @var    right   righmost index of the array to consider
/// @var    n       index of the element of the sorted list to retrun.
/// @return value of the n-th entry of a sorted list.
int quickSelect(int* list, int left, int right, int n);

/// @brief  sorts the elements between left and right if they are bigger or smaller than the value at pivotIndex. Part of the quickselect algorithm(Rosetta code!)
/// @var    list    pointer to the list to partition
/// @var    left    index of the leftmost element  of the partition
/// @var    right   index of the rightmost element of the partition
/// @var    pivotIndex  index of a random point
int quickPartition(int* list, int left, int right, int pivotIndex);

/*********************************
 * IntListMedian implementation *
 *********************************/
int intListMedian(IntList list)
{
    if(list.numElements)
        return quickSelect(list.list,0,list.numElements-1,list.numElements/2);
    else
        return 0;
}

/***********************************
 * Helper function implementations *
 ***********************************/
int quickSelect(int* list, int left, int right, int n)
{
    if (left == right)
        return list[left];

    int pivotIndex = (left + right)/2;
    pivotIndex = quickPartition(list, left, right, pivotIndex);
    if(n == pivotIndex)
        return list[n];
    else if(n < pivotIndex)
        return quickSelect(list, left, pivotIndex - 1, n);
    else
        return quickSelect(list, pivotIndex + 1, right, n);
}

int quickPartition(int* list, int left, int right, int pivotIndex)
{
    int storeIndex, tmp, i;
    int pivotValue = list[pivotIndex];
    list[pivotIndex] = list[right];
    list[right] = pivotValue;
    storeIndex = left;
    for(i=left;i<right;++i){
        if(list[i] < pivotValue){
            tmp = list[storeIndex];
            list[storeIndex] = list[i];
            list[i] = tmp;
            ++storeIndex;
        }
    }
    tmp = list[right];
    list[right] = list[storeIndex];
    list[storeIndex] = tmp;
    return storeIndex;
}

#ifdef __cplusplus
}
#endif

#pragma GCC pop_options
